#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include "ipt_IPID.h"

static struct option opts[] = {
	{"random", 0, 0, 'r'},
	{"incremental", 1, 0, 'i'},
	{0}
};

static void help(void) {
	printf("IPID target v%s options\n", IPTABLES_VERSION);
}

static void init(struct ipt_entry_target *t, unsigned int *nfcache) {
}

static int parse(int c, char **argv, int invert, unsigned int *flags,
		const struct ipt_entry *entry, 
		struct ipt_entry_target **target) {
	struct ipt_ipid_target_info * ipid_info = (struct ipt_ipid_target_info *)(*target)->data;
	switch(c) {
		case 'r':
			ipid_info->mode |= IPID_MODE_RANDOM;
			break;
		case 'i':
			ipid_info->mode |= IPID_MODE_INCREMENTAL;
			ipid_info->step = atoi(optarg);
			break;
		default:
			return 0;
	}
	*flags = ipid_info->mode;
	return 1;
}

static void final_check(unsigned int flags) {
	if(!(flags&IPID_MODE_RANDOM) && !(flags&IPID_MODE_INCREMENTAL))
		exit_error(PARAMETER_PROBLEM, "You have to chose an algorithm\n");
	if(flags&IPID_MODE_RANDOM && flags&IPID_MODE_INCREMENTAL)
		exit_error(PARAMETER_PROBLEM, "You have to chose ONE algorithm\n");
}

static void print(const struct ipt_ip *ip,
		const struct ipt_entry_target *target,
		int numeric) {
}

static void save(const struct ipt_ip *ip,
		const struct ipt_entry_target *target) {
}

static struct iptables_target ipid_target = {
	.next = NULL,
	.name = "IPID",
	.version = IPTABLES_VERSION,
	.size = IPT_ALIGN(sizeof(struct ipt_ipid_target_info)),
	.userspacesize = IPT_ALIGN(sizeof(struct ipt_ipid_target_info)),
	.help = &help,
	.init = &init,
	.parse = &parse,
	.final_check = &final_check,
	.print = &print,
	.save = &save,
	.extra_opts = opts
};

void _init(void) {
	register_target(&ipid_target);
}

